'''
Created on Dec, 2016

@author: jwi

This script performs an energy balance decomposition of local effects in the MPI-ESM.
For conversion of all three surface properties, only surface albedo and only surface roughness.

'''
import os
import numpy as np
import xarray as xr
from copy import deepcopy as cp

#data analysis

#import sys
#import imp
#imp.reload(sys.modules['separate_local_nonlocal3'])
from separate_local_nonlocal3 import separate_local_nonlocal
from Sealandmask_lat_lon import Sealandmask_lat_lon
from scipy.interpolate import griddata

#plot
import pickle #for importing colorbar
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib import gridspec
from mpl_toolkits.basemap import Basemap
from mpl_toolkits import basemap
from mpl_toolkits.basemap import interp
import numpy.ma as ma
#

#imp.reload(sys.modules['Sealandmask_lat_lon'])

'''
--- SECTION 1: load netcdf data (model)------------------------------------------------------------------------------------------
'''
#local effects in model (from 3of4 simulation)
varnames=['var175']
timespan='2000-2199'
#idir='/scratch/local1/m300344/PowerFolders/paper03/data/'
idir='~/paper_jwi/2017_paper03/data/'
data1={}
expids=['jwi0048_1','jwi0044_2']
for varname in varnames:
    for expid in expids:
        data1[expid+'_'+varname]=np.squeeze(xr.open_dataset(idir+'selcode_'+expid+'_echam6_BOT_timmean_'+timespan+'.nc')[varname])
    data1['3of4'+'_'+varname]=data1['jwi0044_2'+'_'+varname].values-data1['jwi0048_1'+'_'+varname].values
#    data1['z0'+'_'+varname]=data1['jwi0053_1'+'_'+varname].values-data1['jwi0048_1'+'_'+varname].values
#    data1['albedo'+'_'+varname]=data1['jwi0043_1'+'_'+varname].values-data1['jwi0048_1'+'_'+varname].values

#%%
'''
--- SECTION 2: separate local from non-local effects----------------------------------------------------------------------
'''
lcc_patterns=['3of4']

local={}
local_interped={}
nonlocal={}
for varname in varnames:
    for lcc_pattern in lcc_patterns:
        local_interped[lcc_pattern+'_'+varname],local[lcc_pattern+'_'+varname],nonlocal[lcc_pattern+'_'+varname]=np.zeros((96, 192)),np.zeros((96, 192)),np.zeros((96, 192))
        local_interped[lcc_pattern+'_'+varname],local[lcc_pattern+'_'+varname],nonlocal[lcc_pattern+'_'+varname]=separate_local_nonlocal(data1[lcc_pattern+'_'+varname], lcc_pattern)
#        local_interped['z0_'+varname],local['z0_'+varname],nonlocal['z0_'+varname]=np.zeros((96, 192)),np.zeros((96, 192)),np.zeros((96, 192))
#        local_interped['z0_'+varname],local['z0_'+varname],nonlocal['z0_'+varname]=separate_local_nonlocal(data1['z0_'+varname], lcc_pattern)
#        local_interped['albedo_'+varname],local['albedo_'+varname],nonlocal['albedo_'+varname]=np.zeros((96, 192)),np.zeros((96, 192)),np.zeros((96, 192))
#        local_interped['albedo_'+varname],local['albedo_'+varname],nonlocal['albedo_'+varname]=separate_local_nonlocal(data1['albedo_'+varname], lcc_pattern)

#local_interped['3of4_var175']=cp(local_interped['3of4'+'_var175'])*(data1['jwi0048_1'+'_var176'].values-data1['jwi0048_1'+'_var204'].values) 
#nonlocal['3of4_var175']=cp(nonlocal['3of4'+'_var175'])*(data1['jwi0048_1'+'_var176'].values-data1['jwi0048_1'+'_var204'].values) 

#local_interped['z0_var175']=cp(local_interped['z0'+'_var175'])*(data1['jwi0048_1'+'_var176'].values-data1['jwi0048_1'+'_var204'].values) 
#nonlocal['3of4_var175']=cp(nonlocal['3of4'+'_var175'])*(data1['jwi0048_1'+'_var176'].values-data1['jwi0048_1'+'_var204'].values) 
#
#local_interped['albedo_var175']=cp(local_interped['albedo'+'_var175'])*(data1['jwi0048_1'+'_var176'].values-data1['jwi0048_1'+'_var204'].values) 
#nonlocal['3of4_var175']=cp(nonlocal['3of4'+'_var175'])*(data1['jwi0048_1'+'_var176'].values-data1['jwi0048_1'+'_var204'].values) 


    
#%%
'''
--- SECTION 5: save data ------------------------------------------------------------------------------------------
'''
np.save('/home/jwi/paper_jwi/2018_paper05/data/save/albedo_change',local_interped[lcc_pattern+'_'+varname])

#%%
#---------PREPARE plots-----------------
##get colorbar
#os.chdir('/home/jwi/paper_jwi/2017_paper03/analysis/scripts')
#fileObject = open('bwr_jwi','r')
#cmap_bwr_jwi = pickle.load(fileObject)
#fileObject.close()

import matplotlib.cm as cm
cmap_viridis=cm.viridis

#bounds=(-5,-4,-3,-2,-1.5,-1,-0.5,-0.1,0.1,0.5,1,1.5,2,3.0,4,5)
bounds=np.arange(0,0.36,0.03)

cmap_viridis.set_bad('grey',0.7)
norm=mpl.colors.BoundaryNorm(bounds, cmap_viridis.N)

#lats and lons
lon=data1[expid+'_'+varname].lon.values
lat=data1[expid+'_'+varname].lat.values
meshlon=lon-1.2
meshlat=lat+0.6

plotmax=0.95*max(bounds)

meshlon, meshlat= np.meshgrid(meshlon, meshlat)

slm,lat,lon,slm1=Sealandmask_lat_lon().slm,Sealandmask_lat_lon().lat,Sealandmask_lat_lon().lon,Sealandmask_lat_lon().slm1
local_interped[lcc_pattern+'_'+varname]=ma.masked_array(local_interped[lcc_pattern+'_'+varname],slm1==0)

##needed for stippling:
#x,y={},{}
#m = Basemap(projection='robin',lon_0=0,resolution='c')
#for expid in expids[1:]:
#    x[expid],y[expid]=m(lon[np.where(data[expid+'_diff_signif'])[1]]-0.2,lat[np.where(data[expid+'_diff_signif'])[0]]+0.2)
# 


#%%
#---------PLOT-----------------

fig = plt.figure(figsize=(8,6))
gs= gridspec.GridSpec(2, 1, height_ratios=(25, 1.5))
#
axes = plt.subplot(gs[0,0])
m = Basemap(projection='robin',lon_0=0,resolution='c')
m.drawcoastlines(linewidth=0.5)
plot=m.pcolormesh(meshlon, meshlat, local_interped[lcc_pattern+'_'+varname], latlon=True, cmap=cmap_viridis, norm=norm,vmin=-plotmax, vmax=plotmax,rasterized=True,zorder=1)
#plt.annotate('a)', xy=(0.05, 0.9), xycoords='axes fraction',fontsize=21)
#m.scatter(x['jwi0046_1'],y['jwi0046_1'],marker='.',s=10,c='black',linewidth=0.2,alpha=0.6)
#local_text=axes.set_title('1/4 \n',fontsize=21)
local_text.set_verticalalignment('center')
#axes.set_ylabel('1/4',fontsize=21)
#
#
axes = plt.subplot(gs[1,:])
cb=mpl.colorbar.ColorbarBase(axes, cmap=cmap_viridis, norm=norm, ticks=bounds,spacing='uniform', boundaries=bounds, orientation='horizontal')
cb.ax.tick_params(labelsize=17)
cb.set_label('$\Delta$ surface albedo, local []',fontsize=21)

plt.tight_layout()

#pfile='/home/jwi/paper_jwi/2017_paper03/analysis/plots/signif_S6.png'
pfile='/home/jwi/paper_jwi/2017_paper03/analysis/plots/delta_albedo_local.pdf'
fig.savefig(pfile,dpi=180, bbox_inches='tight')
#plt.close()



np.save('/home/jwi/ownCloud/paper_jwi/2018_paper05/data/save/delta_albedo_local',local_interped[lcc_pattern+'_'+varname].data)

