'''
Created on Dec, 2016

@author: jwi

This script separates local and non-local effects for different extents of deforestation. It creates plots of maps, and a x-y plot with deforestation extent vs. delta <varname>.
'''
import numpy as np
#import Nio
import xarray as xr
from copy import deepcopy as cp

#data analysis
#import sys
#import imp
from separate_local_nonlocal3 import separate_local_nonlocal
from Sealandmask_lat_lon import Sealandmask_lat_lon
from timeseries_statistics import conficence_interval_mean

#plot
#import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib import gridspec
#from mpl_toolkits.basemap import Basemap
#import numpy.ma as ma
from mpl_toolkits.basemap import Basemap

#imp.reload(sys.modules['Sealandmask_lat_lon'])

'''
--- SECTION 1: load netcdf data ------------------------------------------------------------------------------------------
'''

varnames=['var169','var146','var147','var176', 'var204', 'var177','var205','var204','var175']
timespan='2000-2199'
idir='/home/jwi/paper_jwi/2017_paper03/data/'
idir1='/home/jwi/paper_jwi/2018_paper05/data/'
data={}
expids=['jwi0048_1','jwi0044_2','jwi0043_1','jwi0053_1']

for varname in varnames:
    for expid in expids:
        data[expid+'_'+varname]=np.squeeze(xr.open_dataset(idir+'selcode_'+expid+'_echam6_BOT_ymonmean_2000-2199.nc')[varname])

    data['extensive'+'_'+varname]=data['jwi0044_2'+'_'+varname].values-data['jwi0048_1'+'_'+varname].values
    data['extensive_smoothforest'+'_'+varname]=data['jwi0053_1'+'_'+varname].values-data['jwi0048_1'+'_'+varname].values
    data['extensive_brightforest'+'_'+varname]=data['jwi0043_1'+'_'+varname].values-data['jwi0048_1'+'_'+varname].values

ntime=data['extensive'+'_'+varname].shape[0]
#%%
'''
--- SECTION 2: separate local from non-local effects----------------------------------------------------------------------
'''
lcc_patterns=['extensive','extensive_smoothforest','extensive_brightforest']

local={}
local_interped={}
nonlocal={}
for varname in varnames:
    for lcc_pattern in lcc_patterns:
        local_interped[lcc_pattern+'_'+varname],local[lcc_pattern+'_'+varname],nonlocal[lcc_pattern+'_'+varname]=np.zeros(data[lcc_pattern+'_'+varname].shape),np.zeros(data[lcc_pattern+'_'+varname].shape),np.zeros(data[lcc_pattern+'_'+varname].shape)
        for mon in np.arange(ntime):
            local_interped[lcc_pattern+'_'+varname][mon],local[lcc_pattern+'_'+varname][mon],nonlocal[lcc_pattern+'_'+varname][mon]=separate_local_nonlocal(data[lcc_pattern+'_'+varname][mon], 'extensive')

#%%    
'''
--- SECTION 3: Make monthly boreal statistics and energy balance decomposition------------------------------------------------------------------------------------------
'''
#convert W/m2 into K
factor1=-1/(5.67e-8*4*(data['jwi0048_1'+'_var169'].values)**3)
for varname in varnames:
    for lcc_pattern in lcc_patterns:
        local_interped[lcc_pattern+'_'+varname+'_K'],local[lcc_pattern+'_'+varname+'_K'],nonlocal[lcc_pattern+'_'+varname+'_K']=factor1*(local_interped[lcc_pattern+'_'+varname],local[lcc_pattern+'_'+varname],nonlocal[lcc_pattern+'_'+varname])

#determine which areas you want to average over
slm,lat,lon,slm1=Sealandmask_lat_lon().slm,Sealandmask_lat_lon().lat,Sealandmask_lat_lon().lon,Sealandmask_lat_lon().slm1
slm1_boreal=cp(slm1)
snow= np.squeeze(xr.open_dataset(idir1+'selcode_jwi0048_1_jsbach_land_ymonmean_2000-2199.nc')['var60'])[2]#snow cover in march
bool_array=snow<0.5
slm1_boreal[bool_array.values==True]=0
slm1_boreal[lat<45,:]=0
boreal_landmeans={}

#average spatially and for each month
plotlines={}
for varname in varnames:
    for lcc_pattern in lcc_patterns:
        boreal_landmeans[lcc_pattern+'_reshaped_local'+'_'+varname]=local[lcc_pattern+'_'+varname].reshape((ntime/12,12,96,192))
        for effect in ['local']:
            boreal_landmeans[lcc_pattern+'_timmean_'+effect+'_'+varname]=np.zeros(12)
            for mon in np.arange(12):
                boreal_landmeans[lcc_pattern+'_timmean_'+effect+'_'+varname][mon]=np.nanmean(boreal_landmeans[lcc_pattern+'_reshaped_'+effect+'_'+varname][:,mon,slm1_boreal==1])    
    #for plotting, close seasonal cycle by adding January at the end.
    for lcc_pattern in lcc_patterns:
        for effect in ['local']:
            for line in ['_timmean']:
                plotlines[lcc_pattern+'_'+effect+line+'_'+varname]=np.zeros(13)
                for mon in np.arange(12):
                    plotlines[lcc_pattern+'_'+effect+line+'_'+varname][mon]= boreal_landmeans[lcc_pattern+line+'_'+effect+'_'+varname][mon]
                plotlines[lcc_pattern+'_'+effect+line+'_'+varname][12]=cp(plotlines[lcc_pattern+'_'+effect+line+'_'+varname][0]) 

#%%
'''
--- SECTION 4: save data------------------------------------------------------------------------------------------
'''
np.save('/home/jwi/paper_jwi/2018_paper05/data/save/ymonmean_lines',plotlines)
np.save('/home/jwi/paper_jwi/2018_paper05/data/save/snow_march_greater50',slm1_boreal)


#%%
'''
--- SECTION 5: PLOT Lines------------------------------------------------------------------------------------------
'''
lw=2 #line width

monthnames=['J','F','M','A','M','J','J','A','S','O','N','D']
colors=['black','limegreen','royalblue','violet']

fig = plt.figure(figsize=(8,8))
gs= gridspec.GridSpec(2, 2)

axes = plt.subplot(gs[0,0])
plt.title('a) Changes in $T_{surf}$ in b)-d)')
plt.plot(np.arange(13),plotlines['extensive_local_timmean_var169'], label='all',color=colors[0],zorder=3, linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_smoothforest_local_timmean_var169'], label='only roughness',color=colors[1],zorder=2, linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_brightforest_local_timmean_var169'], label='only albedo',color=colors[3],zorder=1, linewidth=lw)
#plt.plot(np.arange(13),plotlines['extensive_local_timmean_var169']-plotlines['extensive_brightforest_local_timmean_var169']-plotlines['extensive_smoothforest_local_timmean_var169'], label='residual',color=colors[3],zorder=0, linewidth=lw)
#plt.fill_between(np.arange(13),plotlines['extensive_local_low'],plotlines['extensive_local_high'],facecolor=colors[0],alpha=0.5)
#plt.fill_between(np.arange(13),plotlines['extensive_nonlocal_low'],plotlines['extensive_nonlocal_high'],facecolor=colors[1],alpha=0.5)

plt.xticks(np.arange(12),monthnames)
plt.axhline(y=0,c='lightgrey',zorder=0)
plt.ylim((-2.5,2.5))
plt.xlim((0,12))
#plt.xlabel('Month')
plt.ylabel('$\Delta T\mathrm{_{surf} [K]}$')
plt.legend()



axes = plt.subplot(gs[0,1])
plt.title('b) all')
plt.plot(np.arange(13),plotlines['extensive_local_timmean_var169']+10e12, ls='--',label='$T_{surf}$',color='grey', linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_local_timmean_var169'],color=colors[0], ls='--',linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_local_timmean_var176'], label='net shortwave',color='red', linewidth=lw)
plt.plot(np.arange(13),(plotlines['extensive_local_timmean_var177']-plotlines['extensive_local_timmean_var205']), label='in longwave',color='orange', linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_local_timmean_var146'], label='sensible',color='c', alpha=0.6, linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_local_timmean_var147'], label='latent',color='blue', linewidth=lw)
#plt.fill_between(np.arange(13),plotlines['extensive_local_low'],plotlines['extensive_local_high'],facecolor=colors[0],alpha=0.5)
#plt.fill_between(np.arange(13),plotlines['extensive_nonlocal_low'],plotlines['extensive_nonlocal_high'],facecolor=colors[1],alpha=0.5)

plt.xticks(np.arange(12),monthnames)
plt.axhline(y=0,c='lightgrey',zorder=0)
plt.ylim((-20,20))
plt.xlim((0,12))
#plt.xlabel('Month')
plt.legend()



axes = plt.subplot(gs[1,1])
plt.title('d) only albedo')
plt.plot(np.arange(13),plotlines['extensive_brightforest_local_timmean_var169'], ls='--',label='$T_{surf}$',color=colors[3], linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_brightforest_local_timmean_var176'], label='net shortwave',color='red', linewidth=lw)
plt.plot(np.arange(13),(plotlines['extensive_brightforest_local_timmean_var177']-plotlines['extensive_brightforest_local_timmean_var205']), label='in longwave',color='orange', linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_brightforest_local_timmean_var146'], label='sensible',color='c', alpha=0.6, linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_brightforest_local_timmean_var147'], label='latent',color='blue', linewidth=lw)
#plt.fill_between(np.arange(13),plotlines['extensive_local_low'],plotlines['extensive_local_high'],facecolor=colors[0],alpha=0.5)
#plt.fill_between(np.arange(13),plotlines['extensive_nonlocal_low'],plotlines['extensive_nonlocal_high'],facecolor=colors[1],alpha=0.5)

plt.xticks(np.arange(12),monthnames)
plt.axhline(y=0,c='lightgrey',zorder=0)
plt.ylim((-20,20))
plt.xlim((0,12))
#plt.xlabel('Month')
#plt.ylabel('$\Delta T\mathrm{_{surf} [K]}$')
#plt.legend()



axes = plt.subplot(gs[1,0])
plt.title('c) only roughness')
plt.plot(np.arange(13),plotlines['extensive_smoothforest_local_timmean_var169'], ls='--',label='$T_{surf}$',color=colors[1], linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_smoothforest_local_timmean_var176'], label='net shortwave',color='red', linewidth=lw)
plt.plot(np.arange(13),(plotlines['extensive_smoothforest_local_timmean_var177']-plotlines['extensive_smoothforest_local_timmean_var205']), label='in longwave',color='orange', linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_smoothforest_local_timmean_var146'], label='sensible',color='c', alpha=0.6, linewidth=lw)
plt.plot(np.arange(13),plotlines['extensive_smoothforest_local_timmean_var147'], label='latent',color='blue', linewidth=lw)
#plt.fill_between(np.arange(13),plotlines['extensive_local_low'],plotlines['extensive_local_high'],facecolor=colors[0],alpha=0.5)
#plt.fill_between(np.arange(13),plotlines['extensive_nonlocal_low'],plotlines['extensive_nonlocal_high'],facecolor=colors[1],alpha=0.5)

plt.xticks(np.arange(12),monthnames)
plt.axhline(y=0,c='lightgrey',zorder=0)
plt.ylim((-20,20))
plt.xlim((0,12))
plt.xlabel('Month')
plt.ylabel('$\Delta T\mathrm{_{surf} [K]}$')
#plt.legend()


plt.tight_layout()

fig.savefig('/home/jwi/paper_jwi/2018_paper05/analysis/plots/ymonmean_ebd_z0.pdf',dpi=200, bbox_inches='tight')
fig.savefig('/home/jwi/paper_jwi/2018_paper05/analysis/plots/ymonmean_ebd_z0.png',dpi=200, bbox_inches='tight')
#plt.close()

#%%plot areas with march snow cover > threshold

bounds=np.arange(0,1.1,0.1)

#lats and lons
offset_lon,offset_lat=-1,1
meshlon=lon+offset_lon
meshlat=lat+offset_lat

plotmax=0.95*max(bounds)

meshlon, meshlat= np.meshgrid(meshlon, meshlat)


fig = plt.figure(figsize=(8,15))
gs= gridspec.GridSpec(1,1)

axes = plt.subplot(gs[0,0])
m = Basemap(projection='robin',lon_0=0,resolution='c')
m.drawcoastlines(linewidth=0.5)
plot=m.pcolormesh(meshlon, meshlat, slm1_boreal, latlon=True,vmin=-plotmax, vmax=plotmax,rasterized=True,zorder=1)
#plt.title('roughness forest',fontsize=18)
#axes.set_ylabel('roughness forest', fontsize = 18)
#

plt.tight_layout()
fig.savefig('/home/jwi/paper_jwi/2018_paper05/analysis/plots/snowcover_march_greater50.png',dpi=300, bbox_inches='tight')
fig.savefig('/home/jwi/paper_jwi/2018_paper05/analysis/plots/snowcover_march_greater50.pdf',dpi=300, bbox_inches='tight')
plt.close()
